@@ -20,7 +20,7 @@
 """
 
 version_string = """\
-Nuitka V2.7.16
+Nuitka V2.8
 Copyright (C) 2025 Kay Hayen."""
 
 
