@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """ Templates for the variable handling.
@@ -157,7 +157,7 @@ template_read_mvar_unclear = """\
 """
 
 template_read_locals_dict_with_fallback = """\
-%(to_name)s = DICT_GET_ITEM0(tstate, %(locals_dict)s, %(var_name)s);
+%(to_name)s = %(dict_get_item)s(tstate, %(locals_dict)s, %(var_name)s);
 
 if (%(to_name)s == NULL) {
 %(fallback)s
@@ -169,7 +169,9 @@ template_read_locals_dict_without_fallback = """\
 """
 
 
-template_read_locals_mapping_with_fallback = """\
+# Fallback has no ref, so take one to agree with PyObject_GetItem doing
+# it.
+template_read_locals_mapping_with_fallback_no_ref = """\
 %(to_name)s = PyObject_GetItem(%(locals_dict)s, %(var_name)s);
 
 if (%(to_name)s == NULL) {
@@ -177,6 +179,20 @@ if (%(to_name)s == NULL) {
 %(fallback)s
         Py_INCREF(%(to_name)s);
     } else {
+        FETCH_ERROR_OCCURRED_STATE(tstate, &%(exception_state_name)s);
+        goto %(exception_exit)s;
+    }
+}
+"""
+
+template_read_locals_mapping_with_fallback_ref = """\
+%(to_name)s = PyObject_GetItem(%(locals_dict)s, %(var_name)s);
+
+if (%(to_name)s == NULL) {
+    if (CHECK_AND_CLEAR_KEY_ERROR_OCCURRED(tstate)) {
+%(fallback)s
+    } else {
+        FETCH_ERROR_OCCURRED_STATE(tstate, &%(exception_state_name)s);
         goto %(exception_exit)s;
     }
 }
@@ -276,6 +292,82 @@ if (%(test_code)s) {
 }
 """
 
+template_module_variable_accessor_function = """\
+static PyObject *%(accessor_function_name)s(PyThreadState *tstate) {
+#if %(caching)s
+    PyObject *result;
+
+#if PYTHON_VERSION < 0x3b0
+    static uint64_t dict_version = 0;
+    static PyObject *cache_value = NULL;
+
+    if (moduledict_%(module_identifier)s->ma_version_tag == dict_version) {
+        CHECK_OBJECT_X(cache_value);
+        result = cache_value;
+    } else {
+        dict_version = moduledict_%(module_identifier)s->ma_version_tag;
+
+        result = GET_STRING_DICT_VALUE(moduledict_%(module_identifier)s, (Nuitka_StringObject *)%(var_name)s);
+        cache_value = result;
+    }
+#else
+    static uint32_t dict_keys_version = 0xFFFFFFFF;
+    static Py_ssize_t cache_dk_index = 0;
+
+    PyDictKeysObject *dk = moduledict_%(module_identifier)s->ma_keys;
+    if (likely(DK_IS_UNICODE(dk))) {
+
+#if PYTHON_VERSION >= 0x3c0
+        uint32_t current_dk_version = _Nuitka_PyDictKeys_GetVersionForCurrentState(tstate->interp, dk);
+#else
+        uint32_t current_dk_version = _Nuitka_PyDictKeys_GetVersionForCurrentState(dk);
+#endif
+
+        if (current_dk_version != dict_keys_version) {
+            dict_keys_version = current_dk_version;
+            Py_hash_t hash = Nuitka_Py_unicode_get_hash(%(var_name)s);
+            assert(hash != -1);
+
+            cache_dk_index = Nuitka_Py_unicodekeys_lookup_unicode(dk, %(var_name)s, hash);
+        }
+
+        if (cache_dk_index >= 0) {
+            assert(dk->dk_kind != DICT_KEYS_SPLIT);
+
+            PyDictUnicodeEntry *entries = DK_UNICODE_ENTRIES(dk);
+
+            result = entries[cache_dk_index].me_value;
+
+            if (unlikely(result == NULL)) {
+                Py_hash_t hash = Nuitka_Py_unicode_get_hash(%(var_name)s);
+                assert(hash != -1);
+
+                cache_dk_index = Nuitka_Py_unicodekeys_lookup_unicode(dk, %(var_name)s, hash);
+
+                if (cache_dk_index >= 0) {
+                    result = entries[cache_dk_index].me_value;
+                }
+            }
+        } else {
+            result = NULL;
+        }
+    } else {
+        result = GET_STRING_DICT_VALUE(moduledict_%(module_identifier)s, (Nuitka_StringObject *)%(var_name)s);
+    }
+#endif
+
+#else
+    PyObject *result = GET_STRING_DICT_VALUE(moduledict_%(module_identifier)s, (Nuitka_StringObject *)%(var_name)s);
+#endif
+
+    if (unlikely(result == NULL)) {
+        result = GET_STRING_DICT_VALUE(dict_builtin, (Nuitka_StringObject *)%(var_name)s);
+    }
+
+    return result;
+}
+"""
+
 from . import TemplateDebugWrapper  # isort:skip
 
 TemplateDebugWrapper.checkDebug(globals())
